from tqdm import tqdm
import re
import os
import glob
import ujson
import kerchunk.hdf
from copy import deepcopy
from kerchunk.combine import MultiZarrToZarr
from kerchunk.combine import merge_vars
from kerchunk import df
import zarr
import xarray as xr
import sys
import pandas as pd
from multiprocessing import Pool
import hdf5plugin
from datetime import timedelta
import fsspec
import fsspec.implementations.reference
from fsspec.implementations.reference import LazyReferenceMapper
import shutil
import math

fs = fsspec.filesystem("file")

RUNTRUNK=sys.argv[1]
INTAKEFILE=f"{RUNTRUNK}/erc1011.json"
import intake
cat=intake.open_esm_datastore(INTAKEFILE)
larchive=False

##path trunks:
KERCHUNK_TRUNK = "/work/bm1344/DKRZ/kerchunks_batched"

hdf5plugin.PLUGIN_PATH="/work/ik1017/hdf5plugin/plugins/"
os.environ["HDF5_PLUGIN_PATH"]="/work/ik1017/hdf5plugin/plugins/"

sim_ids=["erc1011"]#,"erc1012"]
last_sim_id = sim_ids[-1]
FILES_PRO_BATCH=50

kerchunks_path = f"{KERCHUNK_TRUNK}/{last_sim_id}"
os.makedirs(kerchunks_path, exist_ok=True)

def get_grid_path(uri):
    downloads_prefix = "http://icon-downloads.mpimet.mpg.de/grids/public/mpim/"
    if uri.startswith(downloads_prefix):
        local_grid_path = os.path.join(
            "/pool/data/ICON/grids/public/mpim",
            uri.split('/')[-1].split('_')[2],
            uri[len(downloads_prefix) :]
        )
        if os.path.isfile(local_grid_path):
            return local_grid_path
        else:
            local_grid_path = os.path.join(
                "/pool/data/ICON/grids/public/mpim",
                uri[len(downloads_prefix) :]
            )
            if os.path.isfile(local_grid_path):
                return local_grid_path            

    raise NotImplementedError(f"no idea about how to get {uri}")

def modify_fill_value_and_time_and_doku(out):
    global FILEFORINTAKE
    name=FILEFORINTAKE.split('/')[-1]
    gl="native"
    if "plev19" in name:
        name=name.replace("plev19","ml")
    if "remap" in  name:
        gl="gr025"
    name=name.split('_remap025')[0]
    updatedict=dict(
        DOKU_Name="EERIE ICON-ESM-ER eerie-control-1950 run",
        DOKU_summary="EERIE ICON-ESM-ER eerie-control-1950 run",
        DOKU_responsible_person="Fabian Wachsmann",
        DOKU_authors="Putrasahan, D.; Kröger, J.; Wachsmann, F.",
        DOKU_License="CC BY 4.0",
        grid_label=gl,
        member_id="r1i1p1f1",
        activity_id="EERIE",
        source_type="AOGCM",
        project_id="EERIE",
        history="deleted for convenience"
    )
    sub=cat.df[cat.df["uri"].str.contains(name)]
    for column in cat.df.columns:
        if column=="variable_id" or "time_m" in column:
            continue
        freqs=sub[column].unique()
        if len(freqs)==1:
            updatedict[column]=freqs[0]

    if not "time_reduction" in updatedict:
        updatedict["time_reduction"]="mean"
    if "inst" in name:
        updatedict["time_reduction"]="inst"
    
    out_ = zarr.open(out)    
    vs=list(out_.array_keys())
    if "time" in vs:
        out_.time[:] = out_.time[:] - 1
        updatedict.update(
            dict(
                time_min=out_.time[0],
                time_max=out_.time[-1]
            )
        )

    if "grid_file_uri" in out_.attrs:
        if "coordinates" in out_.attrs:
            out_.attrs["coordinates"]+=" lon lat"
        else:
            updatedict.update(dict(coordinates="lon lat"))
    out_.attrs.update(updatedict)

    return out

def postprocess(out):
    out = modify_fill_value_and_time_and_doku(out)
    return out

# run_script = '/work/bm1344/k203123/icon-mpim/ec1-5/experiments/erc1011/scripts/erc1006.run'
# sim_id="erc1006"
# data_path = '/work/bm1344/k203123/experiments/erc1006/'

# fixed path which contains the json files generated by kerchunk:
def open_and_load_kerchunk_path(kerchunk_path_single):
    toopen=kerchunk_path_single
    if larchive:
        urlparts=kerchunk_path_single.split("kerchunks_single")
        if len(urlparts)>1:
            fpart="::file://"+urlparts[0]+"/kerchunks_single/"+'/'.join(urlparts[1].split('/')[1:3])+".tar.gz"
            tpart="tar://"+'/'.join(urlparts[1].split('/')[2:])
            toopen=tpart+fpart
            
    try:
        with fsspec.open(toopen, "rb") as f:
                return ujson.load(f)            
    except Exception as e:
        print(e)
        print(toopen)
        raise ValueError("ha")

def batched_mztz(kerchunks,kerchunks_path, pattern, idx,gridname=None):
    #out=LazyReferenceMapper.create(
    #    1000,
    #    "/work/bm0021/k204210/temp.parq",
    #    fsspec.filesystem("file")
    #)
    combined_parq=  os.path.join(kerchunks_path, pattern, f"combined_{idx:06}.parq")
    if os.path.exists(combined_parq):
        #return open_and_load_kerchunk_path(combined_parq.replace(".parq",".json"))
        shutil.rmtree(combined_parq)            
    os.makedirs(combined_parq)

    try:
        combined_jsons=MultiZarrToZarr(
            [open_and_load_kerchunk_path(a) for a in kerchunks],
            concat_dims="time",
            identical_dims=["ncells", "height", "bnds", "height_2", "plev","plev_2"],
            postprocess=postprocess,
            coo_map={"time":"cf:time"},
            #out=out,
            #remote_protocol="memory"
        ).translate()
        #out.flush()
    except Exception as e:
        print([a.replace("/work/bm1344/DKRZ/kerchunks_single/erc1012/","/work/bm1344/DKRZ/ICON/erc1012/postprocessing/interpolation/").replace('.json','.nc') for a in kerchunks])
        raise e
    df.refs_to_dataframe(combined_jsons, combined_parq)
    with open(combined_parq.replace(".parq",".json"),"wb") as f:
        f.write(ujson.dumps(combined_jsons).encode())
    return combined_jsons    

def create_local_grid_and_gridchunk(testfile,kerchunks_path):
    forgridds=xr.open_dataset(
        f"reference::/{testfile}",
        engine="zarr",
        consolidated=False
    )

    if not "grid_file_uri" in forgridds.attrs:
        return None
    gridurl=str(forgridds.attrs["grid_file_uri"])
    gridname=gridurl.split('/')[-1]

    gf=kerchunks_path+"/"+gridname
    kf=gf.replace('.nc','.json')
    if not os.path.isfile(kf):
        print(f"Create grid description {kf}")
        gridds=xr.open_dataset(get_grid_path(gridurl))
        coord_names=["clat","clon","cell_sea_land_mask"]

        coords= gridds[coord_names].rename(
            {
                "cell":"ncells",
                "clon":"lon",
                "clat":"lat"
            }
        )
        coords.attrs={}
        coords.to_netcdf(gf, mode="w")
        gridkc = kerchunk.hdf.SingleHdf5ToZarr(gf)
        with open(kf, "wb") as f:
            f.write(ujson.dumps(gridkc.translate()).encode())

        gridds.close()
    forgridds.close()
    return gridname
        
def kerchunk_by_pattern(pattern,files,pp=False):
    global FILEFORINTAKE

    if len(files) > 1:
        FILEFORINTAKE=files[1].replace('.json','.nc')
        print(f"File to search catalog: {FILEFORINTAKE}")
        
        gridname=None
        if not "remap" in files[0] :
            gridname=create_local_grid_and_gridchunk(files[0], kerchunks_path)

        batched=[
            files[
                (a-1)*FILES_PRO_BATCH:min(a*FILES_PRO_BATCH,len(files))
            ]
            for a in range(1,max(1,math.ceil(len(files)/FILES_PRO_BATCH))+1)
        ]
        
        combined_parq=  os.path.join(kerchunks_path, pattern, f"combined.parq")
        if os.path.exists(combined_parq):
            #return True
            shutil.rmtree(combined_parq)

        if len(
            glob.glob(
                os.path.join(kerchunks_path, pattern, f"combined_*.parq")
            )
        )== len(batched):
            print("already kerchunked all")
            #return True
        single_parquets=[]
        print(len(batched))
        with Pool(min(4,len(batched))) as pool:
            single_parquets+=pool.starmap(
                batched_mztz,[(b,kerchunks_path,pattern, idx,gridname) for idx,b in enumerate(batched)]
            )

        combined_jsons=MultiZarrToZarr(
            single_parquets,
            concat_dims="time",
            coo_map={"time":"cf:time"},
            identical_dims=["ncells", "height", "bnds", "height_2", "plev","plev_2"],
        ).translate()
        if gridname:
            gridout=open_and_load_kerchunk_path(
                kerchunks_path+"/"+gridname.replace('.nc','.json'),
            )
            combined_jsons=merge_vars(
                [gridout,combined_jsons]
            )


        df.refs_to_dataframe(combined_jsons, combined_parq)

kerchunks_path_single_last=f"{KERCHUNK_TRUNK.replace('batched','single')}/{last_sim_id}"
patterns = sorted(
    [
        a.replace('.tar.gz','')
        for a in os.listdir(kerchunks_path_single_last)
    ]
)

for a,pattern in tqdm(enumerate(patterns)):
    #if a < 38:
    #    continue
    #if not "oce_ml_1d_mean" in pattern:# and not "oce_2d_1mth_mean_remap025" in pattern:
    #    continue
    print(
        "----------------------\n"
        f"for pattern {pattern} in {kerchunks_path_single_last}\n"
    )
    files = []
    pp=False
    for sim_id in sim_ids:
        kerchunks_path_single = f"{KERCHUNK_TRUNK.replace('batched','single')}/{sim_id}"

        files+=sorted(
            glob.glob(
                os.path.join(
                    kerchunks_path_single, pattern, '*','*.json'
                ),
                recursive=True
            )
        )
        if not files:
            pp=True
            files+=sorted(
                glob.glob(
                    os.path.join(
                        kerchunks_path_single, pattern, '*.json'
                    ),
                    recursive=True
                )
            )
    if not files:
        print("no files were found")
    else:
        print(
            f"{len(files)} files were found\n"
        )
        kerchunk_by_pattern(
            pattern, files,pp
        )
