import tqdm
import re
import os
import glob
import json
import kerchunk.hdf
from copy import deepcopy
import sys
from multiprocessing import Pool
import hdf5plugin

RUNTRUNK=sys.argv[1]

# Provide the run script as argument one:
run_script = f"{RUNTRUNK}/erc1011.run"
#init = "19510101T000000Z.nc"
init = "195001"
l_skiplastfile=False

##path trunks:
KERCHUNK_TRUNK = "/work/bm1344/DKRZ/kerchunks_single"

hdf5plugin.PLUGIN_PATH="/work/ik1017/hdf5plugin/plugins/"
os.environ["HDF5_PLUGIN_PATH"]="/work/ik1017/hdf5plugin/plugins/"

# Hard code variables
##pattern matching:
PPTRUNK="/work/bm1344/DKRZ/ICON/"
EXPNAME_STRING = r"export EXPNAMEs*=\s*(\S+)"
EXPDIR_STRING = r"EXPDIRs*=\s*(\S+)"
#'output_filename' is also hard coded

def grep_list(filename, pattern):
    matches = []
    with open(run_script, "r") as file:
        for line in file:
            match = re.search(pattern, line)
            if match:
                matches.append(match.group(1))
    return matches

sim_id = grep_list(run_script, EXPNAME_STRING)[0].strip()
kerchunks_path = f"{KERCHUNK_TRUNK}/{sim_id}"
os.makedirs(kerchunks_path, exist_ok=True)

pppatterns=None
if os.path.isdir(f"{PPTRUNK}/{sim_id}/postprocessing/interpolation"):
    pppatterns=[
        pppattern
        for pppattern in os.listdir(
            f"{PPTRUNK}/{sim_id}/postprocessing/interpolation"
        )
    ]
data_path = grep_list(run_script, EXPDIR_STRING)[0].strip().split("run")[0]

#import asyncio

# run_script = '/work/bm1344/k203123/icon-mpim/ec1-5/experiments/erc1011/scripts/erc1006.run'
# sim_id="erc1006"
# data_path = '/work/bm1344/k203123/experiments/erc1006/'

# fixed path which contains the json files generated by kerchunk:

print(
    f"Start to kerchunk data from data path {data_path} \n"
    f"from simulation id {sim_id} \n"
    f"into kerchunk directory {kerchunks_path} \n"
)
suffix = "run_*"
data_path_pattern = os.path.join(data_path, suffix)

# filename_construct_patterns are defined in run_script:
filename_construct_pattern = r"output_filename\s*=\s*(\S+)"

# List to store the matching lines
filename_construct_patterns = grep_list(run_script, filename_construct_pattern)
filename_construct_patterns = [p.replace("'", "*") for p in filename_construct_patterns]
# Now you have a list `matched_lines` containing the matching output filenames
print(
    "-------------------------\n"
    "The following filename construct patterns were found:\n"
    f"{filename_construct_patterns}"
)

#async

def open_and_write_kerchunk(file,kf):
    h5chunks = kerchunk.hdf.SingleHdf5ToZarr(file)
    with open(kf, "wb") as f:
        f.write(json.dumps(h5chunks.translate()).encode())

def map_to_kerchunk(file, depth, kerchunk_path):
    kerchunk_trunk = deepcopy(kerchunk_path)
    splitted_file = file.split('/')
    for a in range(-depth,-1):
        kerchunk_trunk = os.path.join(
            kerchunk_trunk,splitted_file[a]
        )
    os.makedirs(kerchunk_trunk,exist_ok=True)
    kerchunk_file=os.path.join(kerchunk_trunk,splitted_file[-1].replace('.nc','.json'))
    if not os.path.isfile(kerchunk_file):
        return kerchunk_file
    return None

def filter_files(files, pp):
    global l_skiplastfile, init
    fnames=[f.split('/')[-1] for f in files]
    skip_idx = 0
    length = len(files)
    while init in fnames:
        if length > skip_idx:
            skip_idx +=1
        else:
            break
    
    files=files[skip_idx:]
    if files:
        files = [ f for f in files if not ".bck/" in f ]
        if not pp:
            files = [ f for f in files if not "_remap" in f ]
        if len(files) > 1 and l_skiplastfile:
            files = files[0:-1]
            if "1d_" in pattern:
                files = files[0:-1]
    
    if files:
        if "erc1011" in run_script:
            if "atm_2d_1d_min" in files[0] or "atm_2d_1d_max" in files[0]:
                files = [ 
                    f
                    for f in files
                    if not any(f"_{a}" in f for a in range(2002,2009))
                ]
            elif "atm_plev_1d_mean" in files[0]:
                files = [ 
                    f
                    for f in files
                    if not any(f"_{a}" in f for a in range(2002,2008))
                ]            
            files = [ 
                f            
                for f in files
                if not f"run_204" in f
            ]
        if "erc1007" in run_script:
            files = [ 
                f            
                for f in files
                if not f"run_1996" in f
            ]
    return files

def kerchunk_by_pattern(pattern,files,depth,pp=False):
    kerchunks = []
    allfiles=[]
#    files = sorted(glob.glob(os.path.join(data_path_pattern, "**", pattern), recursive=True))
    print(os.path.join(data_path_pattern, pattern))
    if files:
        files=filter_files(files, pp)
    else:
        print(f"Could not find files for pattern {pattern}")
    if not files:
        return True
    #if "_1mth_" in pattern:

    kerchunk_files=[]
    pattern_nostar = pattern.replace("*", "")
    kerchunk_path = os.path.join(kerchunks_path, f"{pattern_nostar}")
    os.makedirs(kerchunk_path, exist_ok=True)
    

    with Pool(64) as pool:
        kerchunk_files = pool.starmap(map_to_kerchunk,[(file, depth, kerchunk_path) for file in files])
    
    newfiles={
        f : kerchunk_files[i]
        for i,f in enumerate(files)
        if kerchunk_files[i]
    }
    if newfiles:
        print(
            "----------------------\n"
            f"for pattern {pattern}\n"
            f"{len(files)} files were found\n"
        )
        allfiles+=newfiles.keys()
        
        with Pool(64) as pool:
            kerchunks+=pool.starmap(open_and_write_kerchunk,[(f,k) for f,k in newfiles.items()])
        #kerchunks += asyncio.run(get_kerchunks(files, kerchunks_path, pattern_nostar))


    if allfiles:
        with open(f"kerchunk_covered_{pattern}.txt","w") as f:
            for af in allfiles:
                f.write(f"{af}\n")

if "erc1011" in run_script:
    if not "*oce_ml_1d_mean*" in filename_construct_patterns:
        filename_construct_patterns.append("*oce_ml_1d_mean*")

for pattern in tqdm.tqdm(filename_construct_patterns):
    if not "oce_ml_1d" in pattern:
        continue
    flist=sorted(
            glob.glob(
                os.path.join(
                    data_path_pattern, pattern
                ),
               recursive=True
            )
        )
                       
    if "oce_5lev" in pattern:
        moreolevs=glob.glob(
                os.path.join(
                    data_path_pattern, "*",pattern
                ),
               recursive=True
            )
        moreolevs=[
            a
            for a in moreolevs
            if not any(a.split('/')[-1] in b for b in flist)
        ]
        flist+=moreolevs
        flist=sorted(flist)


    kerchunk_by_pattern(
        pattern,
        flist,
        2
    )

print(
    "-------------------------\n"
    "The following post-processing file name patterns were found:\n"
    f"{pppatterns}"
)

if pppatterns:
    for pattern in tqdm.tqdm(pppatterns):
        if not "oce_ml_1d" in pattern:
            continue
        #if "oce_2d_1mth_mean" in pattern or "oce_2d_1d_mean" in pattern:
        kerchunk_by_pattern(pattern,sorted(glob.glob(f"{PPTRUNK}/{sim_id}/postprocessing/interpolation/{pattern}/*.nc")),1,pp=True)
        kerchunk_by_pattern(pattern,sorted(glob.glob(f"{PPTRUNK}/{sim_id}/postprocessing/interpolation/{pattern}/*/*.nc", recursive=True)),2,pp=True)